(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var parsers = {
        json: JSON.parse,
        json5: function (source) {
            return require('json5').parse(source);
        },
        yaml: function (source) {
            return require('js-yaml').safeLoad(source);
        }
    };
    function compileTarget(source, CodeBuffer, options) {
        var format = options.format, getNamespace = options.getNamespace;
        var buffer = new CodeBuffer({
            root: options.rootVar,
            filePath: options.filePath,
            target: options.target,
            variable: options.variable,
            afterProcess: options.afterProcess,
            publicPath: options.publicPath
        });
        var json;
        try {
            json = parsers[format](source);
        }
        catch (e) {
            console.error("json parse error: " + e.message);
            return {
                code: '',
                errors: [{
                        code: 1,
                        message: 'json parse error: ' + e.message
                    }]
            };
        }
        if ('$title' in json) {
            delete json.$title;
        }
        if ('$template' in json) {
            delete json.$template;
        }
        if ('$preprocesser' in json) {
            var preprocesser = json.$preprocesser;
            buffer.addPreprocesser(preprocesser, getNamespace);
            delete json.$preprocesser;
        }
        buffer.walk(json);
        var code = buffer.toString();
        return {
            code: code,
            errors: []
        };
    }
    exports.compileTarget = compileTarget;
});
//# sourceMappingURL=common.js.map