(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "./js/index", "./php/index"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var index_1 = require("./js/index");
    var index_2 = require("./php/index");
    var compilers = {
        js: index_1.compile,
        php: index_2.compile
    };
    function compile(options) {
        var errors = [];
        var code = {};
        var source = options.source, rootVar = options.rootVar, _a = options.format, format = _a === void 0 ? 'json' : _a, filePath = options.filePath, variable = options.variable, afterProcess = options.afterProcess, _b = options.target, target = _b === void 0 ? { php: true, js: true } : _b, getNamespace = options.getNamespace;
        if (!Object.keys(target).find(function (key) { return target[key]; })) {
            console.error('please specify at least one target');
            return {
                code: {},
                errors: []
            };
        }
        Object.keys(target).forEach(function (key) {
            if (target[key] && compilers[key]) {
                var res = compilers[key]({
                    source: source,
                    rootVar: rootVar,
                    format: format,
                    filePath: filePath,
                    variable: variable,
                    afterProcess: afterProcess,
                    getNamespace: getNamespace
                });
                code[key] = res.code;
                res.errors && errors.push.apply(errors, res.errors);
            }
        });
        return {
            code: code,
            errors: errors
        };
    }
    exports.default = compile;
});
//# sourceMappingURL=compiler.js.map