var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "../utils", "./json2js"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var utils_1 = require("../utils");
    var json2js_1 = __importDefault(require("./json2js"));
    var CodeType;
    (function (CodeType) {
        CodeType[CodeType["line"] = 0] = "line";
        CodeType[CodeType["raw"] = 1] = "raw";
    })(CodeType || (CodeType = {}));
    var U = 'legoMirrorUtil';
    var tracer = 'ijkhlmn';
    var CodeBuffer = (function () {
        function CodeBuffer(options) {
            this.buffer = [{
                    type: CodeType.line,
                    code: "var $newData"
                }];
            this.trackerIndex = 0;
            this.root = options.root || '$tplData';
            this.filePath = options.filePath;
            this.variable = options.variable;
            this.target = options.target;
            this.afterProcess = options.afterProcess;
            this.publicPath = options.publicPath || '';
            if (this.publicPath && !this.publicPath.endsWith('/')) {
                this.publicPath = this.publicPath + '/';
            }
        }
        CodeBuffer.prototype.walk = function (mirror, parentPath) {
            for (var _i = 0, _a = Object.entries(mirror); _i < _a.length; _i++) {
                var _b = _a[_i], to = _b[0], from = _b[1];
                var toPath = utils_1.getPath(to);
                if (typeof from === 'string') {
                    if (utils_1.isVariable(from)) {
                        this.variableProcess(from, toPath, parentPath);
                    }
                    else if (utils_1.isPointer(from)) {
                        var fromPath = utils_1.getFromPath(from, parentPath);
                        this.fromPath(fromPath.path, toPath, fromPath.parentPath);
                    }
                    else if (utils_1.isMustache(from)) {
                        this.fromMustache(from, toPath, parentPath);
                    }
                    else {
                        this.fromConstant(from, toPath, parentPath);
                    }
                }
                else if (['number', 'boolean'].includes(typeof from)
                    || Array.isArray(from)
                    || !from.$from) {
                    this.fromConstant(from, toPath, parentPath);
                }
                else if (typeof from === 'object' && from.$from) {
                    var _c = from, $from = _c.$from, _d = _c.$action, $action = _d === void 0 ? 'copy' : _d, $data = _c.$data;
                    switch ($action) {
                        case 'copy':
                            if (typeof $data === 'object') {
                                this.addCopy($from, toPath, $data, parentPath);
                            }
                        default:
                            break;
                    }
                }
            }
        };
        CodeBuffer.prototype.addCopy = function (from, to, data, parentPath) {
            var _a = data.$type, $type = _a === void 0 ? 'object' : _a, $mirror = data.$mirror;
            if (!$mirror) {
                utils_1.isPointer(from)
                    ? this.fromPath(utils_1.getPath(from), to, parentPath)
                    : this.fromConstant(from, to, parentPath);
                return;
            }
            if (!parentPath) {
                parentPath = { from: [], to: [] };
            }
            if ($type === 'object' && utils_1.isPointer(from)) {
                var fromPath = utils_1.getFromPath(from, parentPath);
                var _b = this.getParams({ from: fromPath.path, to: to }, true), setter = _b.setter, getter = _b.getter;
                this.buffer.push({
                    type: CodeType.raw,
                    code: "if (" + getter + " != null) {\nif (" + setter + " == null) {"
                });
                this.fromConstant({}, to, fromPath.parentPath);
                this.buffer.push({
                    type: CodeType.raw,
                    code: "}"
                });
                this.walk($mirror, {
                    from: fromPath.parentPath.from.concat(fromPath.path),
                    to: fromPath.parentPath.to.concat(to)
                });
                this.buffer.push({
                    type: CodeType.raw,
                    code: "}"
                });
                return;
            }
            if ($type === 'object') {
                this.fromConstant(from, to, parentPath);
                return;
            }
            if ($type === 'array') {
                var fromPath = utils_1.getFromPath(from, parentPath);
                var _c = this.getParams({
                    from: fromPath.parentPath.from.concat(fromPath.path),
                    to: fromPath.parentPath.to.concat(to)
                }), setter = _c.setter, getter = _c.getter;
                var sGetter = this.getParams({
                    from: fromPath.parentPath.from.concat(fromPath.path),
                    to: fromPath.parentPath.to.concat(to)
                }, true).getter;
                this.buffer.push({
                    type: CodeType.raw,
                    code: "if (" + sGetter + " != null) {\nif (!Array.isArray(" + getter + ")) {\n" + getter + " = [" + getter + "];\n}"
                });
                if (data.$maxItems) {
                    this.buffer.push({
                        type: CodeType.raw,
                        code: getter + " = " + getter + ".slice(0, " + data.$maxItems + ");"
                    });
                }
                var tracerVar = "$" + tracer[this.trackerIndex];
                this.trackerIndex++;
                var sSetter = this.getParams({
                    from: fromPath.parentPath.from.concat(fromPath.path),
                    to: fromPath.parentPath.to.concat(to, [tracerVar])
                }, true).setter;
                this.buffer.push({
                    type: CodeType.raw,
                    code: getter + ".forEach(function($item, " + tracerVar + ") {\nif (" + sSetter + " == null) {"
                });
                this.fromConstant({}, to.concat([tracerVar]), fromPath.parentPath);
                this.buffer.push({
                    type: CodeType.raw,
                    code: "}"
                });
                this.walk($mirror, {
                    from: fromPath.parentPath.from.concat(fromPath.path, [tracerVar]),
                    to: fromPath.parentPath.to.concat(to, [tracerVar])
                });
                this.buffer.push({
                    type: CodeType.raw,
                    code: "})\n}"
                });
            }
        };
        CodeBuffer.prototype.fromConstant = function (constant, to, parentPath) {
            var setter = this.getParams(parentPath).setter;
            var p = {
                type: CodeType.line,
                code: this.transfromSetter(to, setter, json2js_1.default(constant))
            };
            this.buffer.push(p);
        };
        CodeBuffer.prototype.fromMustache = function (from, to, parentPath) {
            var getter = this.getParams(parentPath, true).getter;
            var setter = this.getParams(parentPath).setter;
            var regExp = /{{\s*#\/.+?(?=}})/g;
            var varArray = from.match(regExp);
            var array = [];
            var tmpStr = from;
            for (var i = 0; i < varArray.length; i++) {
                var strArray = tmpStr.split(varArray[i]);
                array.push({
                    type: 'const',
                    val: strArray[0].replace('}}', '')
                });
                array.push({
                    type: 'val',
                    val: varArray[i].replace('{{', '').trim()
                });
                tmpStr = strArray[1];
            }
            var lastConst = tmpStr.replace('}}', '');
            if (lastConst) {
                array.push({
                    type: 'const',
                    val: lastConst
                });
            }
            var code = '';
            for (var i = 0; i < array.length; i++) {
                var item = array[i];
                if (item.type === 'val') {
                    var valCode = "(" + this.transformGetter(utils_1.getPath(item.val), getter, true) + " || '')";
                    code += valCode;
                }
                else {
                    code += json2js_1.default(item.val);
                }
                if (i !== array.length - 1) {
                    code += ' + ';
                }
            }
            var p = {
                type: CodeType.line,
                code: this.transfromSetter(to, setter, code)
            };
            this.buffer.push(p);
        };
        CodeBuffer.prototype.fromPath = function (from, to, parentPath) {
            var _a = this.getParams(parentPath), getter = _a.getter, setter = _a.setter;
            var p = {
                type: CodeType.line,
                code: this.transfromSetter(to, setter, this.transformGetter(from, getter, true))
            };
            this.buffer.push(p);
        };
        CodeBuffer.prototype.variableProcess = function (from, to, parentPath) {
            var _a = this.getParams(parentPath), getter = _a.getter, setter = _a.setter;
            var variable = utils_1.getVariable(from);
            this.buffer.push({
                type: CodeType.line,
                code: this.transfromSetter(to, setter, variable)
            });
        };
        CodeBuffer.prototype.getParams = function (parentPath, isSecure) {
            if (isSecure === void 0) { isSecure = false; }
            var getter = this.root;
            var setter = '$newData';
            if (parentPath && parentPath.from.length > 0) {
                getter = this.transformGetter(parentPath.from, getter, isSecure);
            }
            if (parentPath && parentPath.to.length > 0) {
                setter = this.transformGetter(parentPath.to, setter, isSecure);
            }
            return { getter: getter, setter: setter };
        };
        CodeBuffer.prototype.transformGetter = function (path, variable, isSecure) {
            if (isSecure === void 0) { isSecure = false; }
            var p = '';
            if (isSecure) {
                p = U + ".get(" + variable + ", [" + path.map(function (t) { return /^\$/.test(t) ? t : json2js_1.default(t); }).join(', ') + "])";
            }
            else {
                p = "" + variable + path.map(function (p) { return "[" + (/^\$/.test(p) ? p : json2js_1.default(p)) + "]"; }).join('');
            }
            return p;
        };
        CodeBuffer.prototype.transfromSetter = function (to, setter, content) {
            return setter + " = " + U + ".set(" + setter + ", [" + to.map(function (t) { return /^\$/.test(t) ? t : json2js_1.default(t); }).join(', ') + "], " + content + ")";
        };
        CodeBuffer.prototype.addPreprocesser = function (preprocesser) {
            this.buffer.push({
                type: CodeType.raw,
                code: "\n            const {process} = require('" + preprocesser.replace(/\.ts$/, '') + "');\n            " + this.root + " = Object.assign(" + this.root + ", process(" + this.root + "));\n            "
            });
        };
        CodeBuffer.prototype.toString = function () {
            var variable = this.variable ? this.variable + "\n" : '\n';
            var code = this.buffer
                .map(function (a) { return a.code + (a.type === CodeType.line ? ';' : ''); })
                .join('\n');
            return this.target === 'commonjs'
                ? "\nconst " + U + " = require('" + this.publicPath + "json-mirror-compiler/lib/js/runtime/MirrorUtil');\nfunction process(" + this.root + ") {\n    " + variable + "\n    " + code + "\n    return $newData;\n}\nObject.assign(process, " + JSON.stringify(this.afterProcess ? this.afterProcess : {}) + ");\nmodule.exports = process;\n            " : "\n" + code + "\nreturn $newData;\n            ";
        };
        return CodeBuffer;
    }());
    exports.default = CodeBuffer;
});
//# sourceMappingURL=CodeBuffer.js.map