var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "./json2php", "../utils"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var json2php_1 = __importDefault(require("./json2php"));
    var utils_1 = require("../utils");
    var U = 'LegoMirrorUtil';
    var CodeType;
    (function (CodeType) {
        CodeType[CodeType["line"] = 0] = "line";
        CodeType[CodeType["raw"] = 1] = "raw";
    })(CodeType || (CodeType = {}));
    var tracer = 'ijkhlmn';
    var CodeBuffer = (function () {
        function CodeBuffer(options) {
            this.buffer = [{
                    type: CodeType.line,
                    code: '$newData = array()'
                }];
            this.trackerIndex = 0;
            this.root = options.root || '$tplData';
            this.filePath = options.filePath;
        }
        CodeBuffer.prototype.transformGetter = function (path, variable) {
            var p = "" + variable + path.map(function (p) { return "[" + (/^\$/.test(p) ? p : json2php_1.default(p)) + "]"; }).join('');
            return p;
        };
        CodeBuffer.prototype.transfromSetter = function (to, setter, content) {
            return U + "::set(" + setter + ", array(" + to.map(function (t) { return /^\$/.test(t) ? t : json2php_1.default(t); }).join(', ') + "), " + content + ")";
        };
        CodeBuffer.prototype.getParams = function (parentPath) {
            var getter = this.root;
            var setter = '$newData';
            if (parentPath && parentPath.from.length > 0) {
                getter = this.transformGetter(parentPath.from, getter);
            }
            if (parentPath && parentPath.to.length > 0) {
                setter = this.transformGetter(parentPath.to, setter);
            }
            return { getter: getter, setter: setter };
        };
        CodeBuffer.prototype.fromPath = function (from, to, parentPath) {
            var _a = this.getParams(parentPath), getter = _a.getter, setter = _a.setter;
            var p = {
                type: CodeType.line,
                code: this.transfromSetter(to, setter, this.transformGetter(from, getter))
            };
            this.buffer.push(p);
        };
        CodeBuffer.prototype.fromConstant = function (constant, to, parentPath) {
            var setter = this.getParams(parentPath).setter;
            var p = {
                type: CodeType.line,
                code: this.transfromSetter(to, setter, json2php_1.default(constant))
            };
            this.buffer.push(p);
        };
        CodeBuffer.prototype.fromMustache = function (from, to, parentPath) {
            var _a = this.getParams(parentPath), getter = _a.getter, setter = _a.setter;
            var regExp = /{{\s*#\/.+?(?=}})/g;
            var varArray = from.match(regExp);
            var array = [];
            var tmpStr = from;
            for (var i = 0; i < varArray.length; i++) {
                var strArray = tmpStr.split(varArray[i]);
                array.push({
                    type: 'const',
                    val: strArray[0].replace('}}', '')
                });
                array.push({
                    type: 'val',
                    val: varArray[i].replace('{{', '').trim()
                });
                tmpStr = strArray[1];
            }
            var lastConst = tmpStr.replace('}}', '');
            if (lastConst) {
                array.push({
                    type: 'const',
                    val: lastConst
                });
            }
            var code = '';
            for (var i = 0; i < array.length; i++) {
                var item = array[i];
                if (item.type === 'val') {
                    var valCode = this.transformGetter(utils_1.getPath(item.val), getter);
                    code += i !== array.length - 1
                        ? valCode + " . "
                        : valCode;
                }
                else {
                    code += i !== array.length - 1
                        ? "'" + item.val + "' . "
                        : "'" + item.val + "'";
                }
            }
            var p = {
                type: CodeType.line,
                code: this.transfromSetter(to, setter, code)
            };
            this.buffer.push(p);
        };
        CodeBuffer.prototype.addCopy = function (from, to, data, parentPath) {
            var _a = data.$type, $type = _a === void 0 ? 'object' : _a, $mirror = data.$mirror;
            if (!$mirror) {
                utils_1.isPointer(from)
                    ? this.fromPath(utils_1.getPath(from), to, parentPath)
                    : this.fromConstant(from, to, parentPath);
                return;
            }
            if (!parentPath) {
                parentPath = { from: [], to: [] };
            }
            if ($type === 'object' && utils_1.isPointer(from)) {
                var fromPath = utils_1.getFromPath(from, parentPath);
                var _b = this.getParams({ from: fromPath.path, to: to }), setter = _b.setter, getter = _b.getter;
                this.buffer.push({
                    type: CodeType.raw,
                    code: "if (isset(" + getter + ")) {\nif (!isset(" + setter + ")) {"
                });
                this.fromConstant([], to, fromPath.parentPath);
                this.buffer.push({
                    type: CodeType.raw,
                    code: "}"
                });
                this.walk($mirror, {
                    from: fromPath.parentPath.from.concat(fromPath.path),
                    to: fromPath.parentPath.to.concat(to)
                });
                this.buffer.push({
                    type: CodeType.raw,
                    code: "}"
                });
                return;
            }
            if ($type === 'object') {
                this.fromConstant(from, to, parentPath);
                return;
            }
            if ($type === 'array') {
                var fromPath = utils_1.getFromPath(from, parentPath);
                var _c = this.getParams({
                    from: fromPath.parentPath.from.concat(fromPath.path),
                    to: fromPath.parentPath.to.concat(to)
                }), setter = _c.setter, getter = _c.getter;
                this.buffer.push({
                    type: CodeType.raw,
                    code: "if (isset(" + getter + ")) {\nif (!empty(" + getter + ") && !isset(" + getter + "[0])) {\n" + getter + " = array(" + getter + ");\n}"
                });
                if (data.$maxItems) {
                    this.buffer.push({
                        type: CodeType.raw,
                        code: getter + " = array_slice(" + getter + ", 0, " + data.$maxItems + ");"
                    });
                }
                var tracerVar = "$" + tracer[this.trackerIndex];
                this.trackerIndex++;
                this.buffer.push({
                    type: CodeType.raw,
                    code: "foreach (" + getter + " as " + tracerVar + " => $item) {\nif (!isset(" + setter + "[" + tracerVar + "])) {"
                });
                this.fromConstant([], to.concat([tracerVar]), fromPath.parentPath);
                this.buffer.push({
                    type: CodeType.raw,
                    code: "}"
                });
                this.walk($mirror, {
                    from: fromPath.parentPath.from.concat(fromPath.path, [tracerVar]),
                    to: fromPath.parentPath.to.concat(to, [tracerVar])
                });
                this.buffer.push({
                    type: CodeType.raw,
                    code: "}\n}"
                });
            }
        };
        CodeBuffer.prototype.walk = function (mirror, parentPath) {
            for (var _i = 0, _a = Object.entries(mirror); _i < _a.length; _i++) {
                var _b = _a[_i], to = _b[0], from = _b[1];
                var toPath = utils_1.getPath(to);
                if (typeof from === 'string') {
                    if (utils_1.isPointer(from)) {
                        var fromPath = utils_1.getFromPath(from, parentPath);
                        this.fromPath(fromPath.path, toPath, fromPath.parentPath);
                    }
                    else if (utils_1.isMustache(from)) {
                        this.fromMustache(from, toPath, parentPath);
                    }
                    else {
                        this.fromConstant(from, toPath, parentPath);
                    }
                }
                else if (['number', 'boolean'].includes(typeof from)
                    || Array.isArray(from)
                    || !from.$from) {
                    this.fromConstant(from, toPath, parentPath);
                }
                else if (typeof from === 'object' && from.$from) {
                    var _c = from, $from = _c.$from, _d = _c.$action, $action = _d === void 0 ? 'copy' : _d, $data = _c.$data;
                    switch ($action) {
                        case 'copy':
                            if (typeof $data === 'object') {
                                this.addCopy($from, toPath, $data, parentPath);
                            }
                        default:
                            break;
                    }
                }
            }
        };
        CodeBuffer.prototype.addPreprocesser = function (preprocesser, getNamespace) {
            this.buffer.push({
                type: CodeType.raw,
                code: "require_once(dirname(__FILE__) . '/' . " + JSON.stringify(preprocesser.replace(/\.(ts|php)$/, '') + '.php') + ");\n" + this.root + " = array_merge(" + this.root + ", " + getNamespace(preprocesser) + "\\process(" + this.root + "));"
            });
        };
        CodeBuffer.prototype.toString = function () {
            return this.buffer
                .map(function (a) { return a.code + (a.type === CodeType.line ? ';' : ''); })
                .join('\n');
        };
        return CodeBuffer;
    }());
    exports.default = CodeBuffer;
});
//# sourceMappingURL=CodeBuffer.js.map